/*
   gps-time v0.2
 
   by tommy@mix.hive.no 05. Apr'98
   see README.txt for instructions

*/

#include "gps-time.h"

// global 
struct termios oldtio,newtio,oldstdtio,newstdtio;
char fd;

void init(void)
{   
  // open serial-device
  fd = open(MODEMDEVICE, O_RDWR | O_NOCTTY);
  if (fd <0) {perror(MODEMDEVICE); exit(-1); }
 
  // save current settings
  tcgetattr(fd,&oldtio);
 
  // configure modemdevice, see 506-com.h
  newtio.c_cflag = BAUDRATE | CRTSCTS | BITS | CLOCAL | CREAD;
 
  // ignore bytes with parity errors
  newtio.c_iflag = IGNPAR;
 
  // raw output.
  newtio.c_oflag = 0;
 
  // local echo off
  newtio.c_lflag = 0;
 
  // blocking read until 1 char arrives
  newtio.c_cc[VMIN]=1;
  newtio.c_cc[VTIME]=0;
   
  // flush line, use new settings
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);

  // next stop echo and buffering for stdin
  tcgetattr(0,&oldstdtio);
  tcgetattr(0,&newstdtio); /* get working stdtio */
  newstdtio.c_lflag &= ~(ICANON | ECHO);
  tcsetattr(0,TCSANOW,&newstdtio);
   
  // catch SIGQUIT
  signal(SIGINT, (void*)endprog);
}


void endprog(int errcode)
{
   tcsetattr(fd,TCSANOW,&oldtio);   // restore old modem settings
   tcsetattr(0,TCSANOW,&oldstdtio); // restore old tty setings   
   exit(errcode);
}


char *datestr(void)
{
  time_t ti;
  struct tm *t;
  static char buf[4];
  int i;
  
  time(&ti);
  t=localtime(&ti);
  sprintf(buf,"%2d%2d",t->tm_mon+1, t->tm_mday);
  for(i=0;i<strlen(buf);i++)
    if(buf[i]==' ')
      buf[i]='0';

  return buf;
}


int main(int argc,char *argv[])
{
  char c[MAXBUF+1],i,max;
  char tl[3];

  int hour,min,sec;

  init();

  while(1) {
    read(fd,c,1);
   
    // start of NMEA-string?
    if(c[0]=='$') {
      read(fd,c,strlen(NMEASTR));
   
      // correct NMEA-string?
      if(! strcmp(c,NMEASTR)) {

        //get rid of ','
        read(fd,c,1);

        //read MAXBUF next bytes, which is the time
        i=0;max=0;
        while(i<MAXBUF) {
          i+=read(fd,c+i,MAXBUF);
          max++;

          // avoid infinite loop
          if(max>20) break;
        }
        c[MAXBUF]='\0';

        // error ?
        if(i<MAXBUF+1)
          endprog(1);

        tl[2]='\0';
        strncpy(tl,c,2);
        hour=atoi(tl);
        strncpy(tl,c+2*sizeof(char),2);
        min=atoi(tl);
        strncpy(tl,c+4*sizeof(char),2);
        sec=atoi(tl);


        sec+=SECOFFSET;
        min+=MINOFFSET;
        hour+=HOUROFFSET;

        if(sec>59) {
          sec-=60;
          min+=1;
        }
        if(sec<0) {
          sec+=60;
          min-=1;
        }

        if(min>59) {
          min-=60;
          hour+=1;
        }
        if(min<0) {
          min+=60;
          hour-=1;
        }

        if(hour>23)
          hour-=24;
        if(hour<0)
          hour+=24;
        
        sprintf(c,"%s%2d%2d.%2d",datestr(),hour,min,sec);
        for(i=0;i<strlen(c);i++)
          if(c[i]==' ')
            c[i]='0';

        printf("%s",c);
        endprog(0);
      
      }
    }
  }

  return 0;
}
